@@ -72,6 +72,12 @@ class b2evo_captcha
 	//	Default options, can be overridden from the calling code
 	//
 
+	/**
+	 * Secret key to prevent easy cracking of CAPTCHA code based on the image name.  This value MUST be changed in the configuration file before use.
+	 *
+	 **/
+	var $secret_key;
+
 	/**
 	 * Absolute path to a Tempfolder (with trailing slash!). This must be writeable for PHP and also accessible via HTTP, because the image will be stored there.
 	 *
@@ -581,7 +587,12 @@ class b2evo_captcha
 				$res = @unlink($this->get_filename($correct_hash)) ? 'TRUE' : 'FALSE';
 				if($this->debug) echo "\n<br>-Captcha-Debug: Delete image (".$this->get_filename($correct_hash).") returns: ($res)";
 			}
-			$res = (md5($attempt)===$correct_hash) ? 'TRUE' : 'FALSE';
+
+			// refuse to work unless default secret key is changed
+			if ($this->secret_key == 'CHANGE THIS TO SOME CUSTOM (SECRET) VALUE')
+				die('FATAL ERROR: B2EVO SECRET KEY NOT CONFIGURED');
+
+			$res = (md5($attempt . $this->secret_key)===$correct_hash) ? 'TRUE' : 'FALSE';
 			if($this->debug) echo "\n<br>-Captcha-Debug: Comparing public with private key returns: ($res)";
 			return $res == 'TRUE' ? TRUE : FALSE;
 		}
@@ -631,13 +642,18 @@ class b2evo_captcha
 		// this is where the actual text and public hash is generated and stored
 		function generate_keypair()
 		{
+
+			// refuse to work unless default secret key is changed
+			if ($this->secret_key == 'CHANGE THIS TO SOME CUSTOM (SECRET) VALUE')
+				die('FATAL ERROR: B2EVO SECRET KEY NOT CONFIGURED');
+
 			$key = '';
 			$this->chars = mt_rand($this->minchars,$this->maxchars);
 			for($i=0; $i < $this->chars; $i++) {
 				$key .= $this->validchars{mt_rand(0,strlen($this->validchars)-1)};
 			}
 			if(!$this->case_sensitive) $key = strtoupper($key);
-			$this->public_key = md5($key);
+			$this->public_key = md5($key . $this->secret_key);
 			if($this->debug) echo "\n<br>-Captcha-Debug: Generate Keys, private key is: (".$key.')';
 			if($this->debug) echo "\n<br>-Captcha-Debug: Generate Keys, public key is: (".$this->public_key.')';
 			return $key;
@@ -649,7 +665,7 @@ class b2evo_captcha
 		{
 			if(is_file($filename) ? TRUE : touch($filename))
 			{
-				if(is_readable($filename) && is_writable($filename))
+				if(is_readable($filename) && sq_is_writable($filename))
 				{
 					$fp = @fopen($filename, 'r');
 					if($fp)
